=== modified file '.bzrignore'
--- .bzrignore	2005-11-25 04:36:54 +0000
+++ .bzrignore	2005-12-09 21:48:26 +0000
@@ -19,3 +19,4 @@
 ./ID
 ./tags
 .bzr-shelf
+BRANCH-INFO

=== modified file 'NEWS'
--- NEWS	2006-01-31 13:22:51 +0000
+++ NEWS	2006-01-31 15:33:47 +0000
@@ -9,6 +9,15 @@
       reuses them to avoid multiple connections to the same host (like
       sftp did). (Daniel Silverstone)
 
+    * The bzr_man.py file has been removed. To create the man page now,
+      use ./generate_docs.py man. The new program can also create other files.
+      Run "python generate_docs.py --help" for usage information. (Hans
+      Ulrich Niedermann & James Blackwell).
+
+    * Man Page now gives full help (James Blackwell). Help also updated to 
+      reflect user config now being stored in .bazaar (Hans Ulrich
+      Niedermann)
+
   INTERNALS:
   
     * The internal storage of history, and logical branch identity have now

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2006-01-31 14:52:42 +0000
+++ bzrlib/builtins.py	2006-01-31 15:33:47 +0000
@@ -1166,8 +1166,8 @@
 
     Note: export of tree with non-ascii filenames to zip is not supported.
 
-    Supported formats       Autodetected by extension
-    -----------------       -------------------------
+     Supported formats       Autodetected by extension
+     -----------------       -------------------------
          dir                            -
          tar                          .tar
          tbz2                    .tar.bz2, .tbz2
@@ -1390,8 +1390,8 @@
             print config.username()
 
 class cmd_nick(Command):
-    """\
-    Print or set the branch nickname.  
+    """Print or set the branch nickname.  
+
     If unset, the tree root directory name is used as the nickname
     To print the current nickname, execute with no argument.  
     """

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2006-01-30 11:55:34 +0000
+++ bzrlib/tests/__init__.py	2006-01-31 15:33:47 +0000
@@ -680,8 +680,9 @@
                    'bzrlib.tests.test_commit_merge',
                    'bzrlib.tests.test_config',
                    'bzrlib.tests.test_conflicts',
+                   'bzrlib.tests.test_decorators',
                    'bzrlib.tests.test_diff',
-                   'bzrlib.tests.test_decorators',
+                   'bzrlib.tests.test_doc_generate',
                    'bzrlib.tests.test_fetch',
                    'bzrlib.tests.test_fileid_involved',
                    'bzrlib.tests.test_gpg',

=== added file 'bzrlib/tests/test_doc_generate.py'
--- bzrlib/tests/test_doc_generate.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_doc_generate.py	2005-12-24 12:46:52 +0000
@@ -0,0 +1,9 @@
+from bzrlib.tests import TestCaseInTempDir
+from bzrlib.branch import Branch
+import tools.doc_generate
+
+class TestDocGenerate(TestCaseInTempDir):
+    def test_generate_manpage(self):
+        """Simple smoke test for doc_generate"""
+        infogen_mod = tools.doc_generate.get_module("man")
+

=== added file 'generate_docs.py'
--- generate_docs.py	1970-01-01 00:00:00 +0000
+++ generate_docs.py	2006-01-31 15:33:47 +0000
@@ -0,0 +1,93 @@
+#!/usr/bin/python
+
+# Copyright 2005 Canonical Ltd.
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+"""%(prog)s - generate information from built-in bzr help
+
+%(prog)s creates a file with information on bzr in one of
+several different output formats:
+
+    man              man page
+    bash_completion  bash completion script
+    ...
+
+Examples: 
+
+    python2.4 generated-docs.py man
+    python2.4 generated-docs.py bash_completion
+
+Run "%(prog)s --help" for the option reference.
+"""
+
+import sys
+from optparse import OptionParser
+
+import tools.doc_generate
+
+def main(argv):
+    parser = OptionParser(usage="%prog [options] OUTPUT_FORMAT")
+
+    parser.add_option("-s", "--show-filename",
+                      action="store_true", dest="show_filename", default=False,
+                      help="print default filename on stdout")
+
+    parser.add_option("-o", "--output", dest="filename", metavar="FILE",
+                      help="write output to FILE")
+
+    parser.add_option("-b", "--bzr-name",
+                      dest="bzr_name", default="bzr", metavar="EXEC_NAME",
+                      help="name of bzr executable")
+
+    parser.add_option("-e", "--examples",
+                      action="callback", callback=print_extended_help,
+                      help="Examples of ways to call generate_doc")
+
+
+    (options, args) = parser.parse_args(argv)
+
+    if len(args) != 2:
+        parser.print_help()
+        sys.exit(1)
+
+    infogen_type = args[1]
+    infogen_mod = tools.doc_generate.get_module(infogen_type)
+
+    if options.filename:
+        outfilename = options.filename
+    else:
+        outfilename = infogen_mod.get_filename(options)
+
+    if outfilename == "-":
+        outfile = sys.stdout
+    else:
+        outfile = open(outfilename,"w")
+
+    if options.show_filename and (outfilename != "-"):
+        print >>sys.stdout, outfilename
+    
+    infogen_mod.infogen(options, outfile)
+
+def print_extended_help(option, opt, value, parser):
+    """ Program help examples
+
+    Prints out the examples stored in the docstring. 
+
+    """
+    print >>sys.stdout, __doc__ % {"prog":sys.argv[0]}
+    sys.exit(0)
+
+if __name__ == '__main__':
+    main(sys.argv)

=== modified file 'setup.py'
--- setup.py	2005-12-24 02:20:45 +0000
+++ setup.py	2006-01-31 15:33:47 +0000
@@ -73,8 +73,8 @@
     def run(self):
         build.run(self)
 
-        import bzr_man
-        bzr_man.main()
+        import generate_docs
+        generate_docs.main(argv=["bzr", "man"])
 
 ########################
 ## Setup

=== added file 'tools/__init__.py'
=== added directory 'tools/doc_generate'
=== added file 'tools/doc_generate/__init__.py'
--- tools/doc_generate/__init__.py	1970-01-01 00:00:00 +0000
+++ tools/doc_generate/__init__.py	2005-12-12 01:26:27 +0000
@@ -0,0 +1,34 @@
+#!/usr/bin/python
+
+# Copyright 2005 Canonical Ltd.
+# Written by Hans Ulrich Niedermann
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""bzrinitgen/__init__.py - main program for bzr information generation stuff
+
+"""
+
+
+import sys
+
+def get_module(target):
+    mod_name = "tools.doc_generate.autodoc_%s" % (target)
+    mod = __import__(mod_name)
+    components = mod_name.split('.')
+    for comp in components[1:]:
+        mod = getattr(mod, comp)
+    return mod
+

=== added file 'tools/doc_generate/autodoc_bash_completion.py'
--- tools/doc_generate/autodoc_bash_completion.py	1970-01-01 00:00:00 +0000
+++ tools/doc_generate/autodoc_bash_completion.py	2006-01-30 05:42:13 +0000
@@ -0,0 +1,54 @@
+#!/usr/bin/python
+
+# Copyright 2005 Canonical Ltd.
+# Written by Hans Ulrich Niedermann
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"big_bash_completion.py - create bash completion script from built-in bzr help"
+
+import os
+import sys
+import time
+
+import bzrlib
+import bzrlib.help
+import bzrlib.commands
+
+def get_filename(options):
+    return "%s.bash_completion" % (options.bzr_name)
+
+
+def infogen(options, outfile):
+    t = time.time()
+    tt = time.gmtime(t)
+    params = \
+           { "bzrcmd": options.bzr_name,
+             "datestamp": time.strftime("%Y-%m-%d",tt),
+             "timestamp": time.strftime("%Y-%m-%d %H:%M:%S +0000",tt),
+             "version": bzrlib.__version__,
+             }
+
+    outfile.write(preamble % params)
+
+
+preamble = """\
+# bash completion functions for for %(bzrcmd)s (bazaar-ng)
+#
+# Large parts of this file are autogenerated from the internal
+# bazaar-ng documentation and data structures.
+#
+# Generation time: %(timestamp)s
+"""

=== renamed file 'bzr_man.py' => 'tools/doc_generate/autodoc_man.py'
--- bzr_man.py	2005-12-08 12:01:09 +0000
+++ tools/doc_generate/autodoc_man.py	2006-01-30 05:42:13 +0000
@@ -1,7 +1,6 @@
 #!/usr/bin/python
 
-# Copyright (C) 2005 by Hans Ulrich Niedermann
-# Portions Copyright (C) 2005 by Canonical Ltd
+# Copyright 2005 Canonical Ltd.
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,21 +16,46 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-#<<< code taken from bzr (C) Canonical
-
-import os, sys
-
-import bzrlib, bzrlib.help
-
-#>>> code taken from bzr (C) Canonical
-
-#<<< code by HUN
-
+"""big_man.py - create man page from built-in bzr help and static text
+
+TODO:
+  * use usage information instead of simple "bzr foo" in COMMAND OVERVIEW
+  * add command aliases
+"""
+
+import os
+import sys
+import textwrap
 import time
-import re
+
+import bzrlib
+import bzrlib.help
+import bzrlib.commands
+
+def get_filename(options):
+    """Provides name of manpage"""
+    return "%s.1" % (options.bzr_name)
+
+
+def infogen(options, outfile):
+    """Assembles a man page"""
+    t = time.time()
+    tt = time.gmtime(t)
+    params = \
+           { "bzrcmd": options.bzr_name,
+             "datestamp": time.strftime("%Y-%m-%d",tt),
+             "timestamp": time.strftime("%Y-%m-%d %H:%M:%S +0000",tt),
+             "version": bzrlib.__version__,
+             }
+    outfile.write(man_preamble % params)
+    outfile.write(man_escape(man_head % params))
+    outfile.write(man_escape(getcommand_list(params)))
+    outfile.write(man_escape(getcommand_help(params)))
+    outfile.write(man_escape(man_foot % params))
 
 
 def man_escape(string):
+    """Escapes strings for man page compatibility"""
     result = string.replace("\\","\\\\")
     result = result.replace("`","\\`")
     result = result.replace("'","\\'")
@@ -39,96 +63,72 @@
     return result
 
 
-class Parser:
-
-    def parse_line(self, line):
-        pass
-
-
-class CommandListParser(Parser):
-
-    """Parser for output of "bzr help commands".
-
-    The parsed content can then be used to
-    - write a "COMMAND OVERVIEW" section into a man page
-    - provide a list of all commands
-    """
-
-    def __init__(self,params):
-        self.params = params
-        self.command_usage = []
-        self.all_commands = []
-        self.usage_exp = re.compile("([a-z0-9-]+).*")
-        self.descr_exp = re.compile("    ([A-Z].*)\s*")
-        self.state = 0
-        self.command = None
-        self.usage = None
-        self.descr = None
-
-    def parse_line(self, line):
-        m = self.usage_exp.match(line)
-	if line == '':
-		return
-        if m:
-            if self.state == 0:
-                if self.usage:
-                    self.command_usage.append((self.command,self.usage,self.descr))
-                    self.all_commands.append(self.command)
-                self.usage = " ".join(line.split(" ")[1:])
-                self.command = m.groups()[0]
-            else:
-                raise RuntimeError, "matching usage line in state %d" % state
-            self.state = 1
-            return
-        m = self.descr_exp.match(line)
-        if m:
-            if self.state == 1:
-                self.descr = m.groups()[0]
-            else:
-                raise RuntimeError, "matching descr line in state %d" % state
-            self.state = 0
-            return
-        raise RuntimeError, "Cannot parse this line ('%s')." % line
-
-    def end_parse(self):
-        if self.state == 0:
-            if self.usage:
-                self.command_usage.append((self.command,self.usage,self.descr))
-                self.all_commands.append(self.command)
+def command_name_list():
+    """Builds a list of command names from bzrlib"""
+    command_names = bzrlib.commands.builtin_command_names()
+    command_names.sort()
+    return command_names
+
+
+def getcommand_list (params):
+    """Builds summary help for command names in manpage format"""
+    bzrcmd = params["bzrcmd"]
+    output = '.SH "COMMAND OVERVIEW"\n'
+    for cmd_name in command_name_list():
+        cmd_object = bzrlib.commands.get_cmd_object(cmd_name)
+        if cmd_object.hidden:
+            continue
+        cmd_help = cmd_object.help()
+        if cmd_help:
+            firstline = cmd_help.split('\n', 1)[0]
+            usage = bzrlib.help.command_usage(cmd_object)
+            tmp = '.TP\n.B "%s"\n%s\n' % (usage, firstline)
+            output = output + tmp
         else:
-            raise RuntimeError, "ending parse in state %d" % state
-
-    def write_to_manpage(self, outfile):
-        bzrcmd = self.params["bzrcmd"]
-        outfile.write('.SH "COMMAND OVERVIEW"\n')
-        for (command,usage,descr) in self.command_usage:
-            outfile.write('.TP\n.B "%s %s"\n%s\n' % (bzrcmd, usage, descr))
-
-
-class HelpReader:
-
-    def __init__(self, parser):
-        self.parser = parser
-
-    def write(self, data):
-        if data[-1] == '\n':
-            data = data[:-1]
-        for line in data.split('\n'):
-            self.parser.parse_line(line)
-
-
-def write_command_details(params, command, usage, descr, outfile):
-    x = ('.SS "%s %s"\n.B "%s"\n.PP\n.B "Usage:"\n%s %s\n\n' %
-         (params["bzrcmd"],
-          command,
-          descr,
-          params["bzrcmd"],
-          usage))
-    outfile.write(man_escape(x))
+            raise RuntimeError, "Command '%s' has no help text" % (cmd_name)
+    return output
+
+
+def getcommand_help(params):
+    """Shows individual options for a bzr command"""
+    output='.SH "COMMAND REFERENCE"\n'
+    for cmd_name in command_name_list():
+        cmd_object = bzrlib.commands.get_cmd_object(cmd_name)
+        if cmd_object.hidden:
+            continue
+        output = output + format_command(params, cmd_object)
+    return output
+
+
+def format_command (params, cmd):
+    """Provides long help for each public command"""
+    subsection_header = '.SS "%s"\n' % (bzrlib.help.command_usage(cmd))
+    doc = "%s\n" % (cmd.__doc__)
+    docsplit = cmd.__doc__.split('\n')
+    doc = '\n'.join([docsplit[0]] + [line[4:] for line in docsplit[1:]])
+    option_str = ""
+    options = cmd.options()
+    if options:
+        option_str = "\nOptions:\n"
+        for option_name, option in sorted(options.items()):
+            l = '    --' + option_name
+            if option.type is not None:
+                l += ' ' + option.argname.upper()
+            short_name = option.short_name()
+            if short_name:
+                assert len(short_name) == 1
+                l += ', -' + short_name
+            l += (30 - len(l)) * ' ' + option.help
+            # TODO: Split help over multiple lines with
+            # correct indenting and wrapping.
+            wrapped = textwrap.fill(l, initial_indent='',
+                                    subsequent_indent=30*' ')
+            option_str = option_str + wrapped + '\n'       
+    return subsection_header + option_str + "\n" + doc + "\n"
 
 
 man_preamble = """\
-.\\\" Man page for %(bzrcmd)s (bazaar-ng)
+Man page for %(bzrcmd)s (bazaar-ng)
 .\\\"
 .\\\" Large parts of this file are autogenerated from the output of
 .\\\"     \"%(bzrcmd)s help commands\"
@@ -138,8 +138,7 @@
 .\\\"
 """
 
-# The DESCRIPTION text was taken from http://www.bazaar-ng.org/
-# and is thus (C) Canonical
+
 man_head = """\
 .TH bzr 1 "%(datestamp)s" "%(version)s" "bazaar-ng"
 .SH "NAME"
@@ -174,75 +173,27 @@
 is to look for external command.
 .TP
 .I "BZREMAIL"
-E-Mail address of the user. Overrides
-.I "~/.bzr.conf/email" and
-.IR "EMAIL" .
-Example content:
-.I "John Doe <john@example.com>"
+E-Mail address of the user. Overrides default user config.
 .TP
 .I "EMAIL"
-E-Mail address of the user. Overridden by the content of the file
-.I "~/.bzr.conf/email"
-and of the environment variable
-.IR "BZREMAIL" .
+E-Mail address of the user. Overriddes default user config.
 .SH "FILES"
 .TP
-.I "~/.bzr.conf/"
-Directory where all the user\'s settings are stored.
-.TP
-.I "~/.bzr.conf/email"
-Stores name and email address of the user. Overrides content of
-.I "EMAIL"
-environment variable. Example content:
-.I "John Doe <john@example.com>"
+.I "~/.bazaar/bazaar.conf/"
+Contains the default user config. Only one section, [DEFAULT] is allowed. A 
+typical default config file may be similiar to:
+.br
+.br
+.B [DEFAULT]
+.br
+.B email=John Doe <jdoe@isp.com>
 .SH "SEE ALSO"
+.UR http://bazaar.canonical.com/
+.BR http://bazaar.canonical.com/,
 .UR http://www.bazaar-ng.org/
-.BR http://www.bazaar-ng.org/,
-.UR http://www.bazaar-ng.org/doc/
-.BR http://www.bazaar-ng.org/doc/
+.BR http://www.bazaar-ng.org/
 """
 
-def main(args=[]):
-    """ main function
-    :param  args:   command-line arguments (sys.argv[1:])
-    :type   args:   list
-    """
-    t = time.time()
-    tt = time.gmtime(t)
-    params = \
-           { "bzrcmd": "bzr",
-             "datestamp": time.strftime("%Y-%m-%d",tt),
-             "timestamp": time.strftime("%Y-%m-%d %H:%M:%S +0000",tt),
-             "version": bzrlib.__version__,
-             }
-
-    clp = CommandListParser(params)
-    bzrlib.help.help("commands", outfile=HelpReader(clp))
-    clp.end_parse()
-
-    filename = "bzr.1"
-    if len(args) == 1:
-        filename = args[0]
-    if filename == "-":
-        outfile = sys.stdout
-    else:
-        outfile = open(filename,"w")
-
-    outfile.write(man_preamble % params)
-    outfile.write(man_escape(man_head % params))
-    clp.write_to_manpage(outfile)
-
-    # FIXME:
-    #   This doesn't do more than the summary so far.
-    #outfile.write('.SH "DETAILED COMMAND DESCRIPTION"\n')
-    #for (command,usage,descr) in clp.command_usage:
-    #    write_command_details(params, command, usage, descr, outfile = outfile)
-
-    outfile.write(man_escape(man_foot % params))
-
-
 if __name__ == '__main__':
     main(sys.argv[1:])
 
-
-#>>> code by HUN

